package java.lang;

import rte.SClassDesc;

public class Object {
  public SClassDesc _r_type; //fixed by compiler: reloc-position 0
}

package java.lang;

public class String {
  public char[] value;
  public int count;

  public int length() {
    return count;
  }
}

package rte;

public class SArray {
  public int length, _r_dim, _r_stdType;
  public Object _r_unitType;
}

package rte;

public class SClassDesc {
  public SClassDesc parent;
  public SIntfMap implementations;
}

package rte;

public class SIntfDesc {
}

package rte;

public class SIntfMap {
  public SIntfDesc owner;
  public SIntfMap next;
}

package rte;

public class SMthdBlock {
}

package rte;

public class DynamicRuntime {
  public static int objCnt;
  
  private final static int MEMBLOCK = 1024*1024; //1 MB per block
  private final static int BLOCKMAX = 64*1024;   //allocate objects up to 64 KB in memblock
  private static int nextFreeAddr, memFree;
  
  public static Object newInstance(int scalarSize, int relocEntries, SClassDesc type) {
    int addr, rs, size;
    Object me;
    
    //get informations
    rs=relocEntries*MAGIC.ptrSize;
    scalarSize=(scalarSize+MAGIC.ptrSize-1)&~(MAGIC.ptrSize-1);
    size=rs+scalarSize;
    if (size>BLOCKMAX) addr=allocMem(size); //allocate special block
    else { //allocate inside the pool
      if (size>memFree) {
        nextFreeAddr=allocMem(MEMBLOCK);
        memFree=MEMBLOCK;
      }
      addr=nextFreeAddr;
      memFree-=size;
      nextFreeAddr+=size;
    }
    //prepare object
    setMem32(addr, size>>2, 0); //clear memory
    me=MAGIC.cast2Obj(addr+rs); //place object
    me._r_type=type; //set type
    objCnt++;
    return me;
  }
  
  private static int allocMem(int size) {
    int result;
    
    result=brk(0);
    if (result<=0) {
      kernel.Viewer.print("Error allocating memory");
      exit(-1);
    }
    size=(size+0xFFF)&0xFFFFF000;
    brk(result+size);
    return result;
  }
  
  private static void setMem32(int addr, int cnt, int val) {
    MAGIC.inline(0x57);             //push edi
    MAGIC.inline(0xFC);             //cld
    MAGIC.inline(0x8B, 0x7D, 0x10); //mov edi,[ebp+16]
    MAGIC.inline(0x8B, 0x45, 0x08); //mov eax,[ebp+8]
    MAGIC.inline(0x8B, 0x4D, 0x0C); //mov ecx,[ebp+12]
    MAGIC.inline(0xF3, 0xAB);       //rep stosd
    MAGIC.inline(0x5F);             //pop edi
  }
  
  public static SArray newArray(int length, int arrDim, int entrySize, int stdType, Object unitType) {
    int scS, rlE;
    SArray me;
    
    scS=MAGIC.getInstScalarSize("SArray");
    rlE=MAGIC.getInstRelocEntries("SArray");
    if (arrDim>1 || entrySize<0) rlE+=length; //objects => rlE will become bigger
    else scS+=length*entrySize; //entrySize>0 => scS will become bigger
    me=(SArray)newInstance(scS, rlE, MAGIC.clssDesc("SArray"));
    me.length=length;
    me._r_dim=arrDim;
    me._r_stdType=stdType;
    me._r_unitType=unitType;
    return me;
  }
  
  public static void newMultArray(SArray[] parent, int curLevel, int destLevel,
      int length, int arrDim, int entrySize, int stdType, Object unitType) {
    int i;
    
    if (curLevel+1<destLevel) { //step down one level
      curLevel++;
      for (i=0; i<parent.length; i++) {
        newMultArray((SArray[])((Object)parent[i]), curLevel, destLevel,
            length, arrDim, entrySize, stdType, unitType);
      }
    }
    else { //create the new entries
      destLevel=arrDim-curLevel;
      for (i=0; i<parent.length; i++) {
        parent[i]=newArray(length, destLevel, entrySize, stdType, unitType);
      }
    }
  }
  
  public static boolean isInstance(Object o, SClassDesc dest, boolean asCast) {
    SClassDesc check;
    
    if (o==null) {
      if (asCast) return true; //null matches all
      return false; //null is not an instance
    }
    check=o._r_type;
    while (check!=null) {
      if (check==dest) return true;
      check=check.parent;
    }
    if (asCast) exit(-1);
    return false;
  }
  
  public static SIntfMap isImplementation(Object o, SIntfDesc dest, boolean asCast) {
    SIntfMap check;
    
    if (o==null) return null;
    check=o._r_type.implementations;
    while (check!=null) {
      if (check.owner==dest) return check;
      check=check.next;
    }
    if (asCast) exit(-1);
    return null;
  }
  
  public static boolean isArray(SArray o, int stdType, Object unitType, int arrDim, boolean asCast) {
    SClassDesc clss;
    
    //in fact o is of type "Object", _r_type has to be checked below - but this check is faster than "instanceof" and conversion
    if (o==null) {
      if (asCast) return true; //null matches all
      return false; //null is not an instance
    }
    if (o._r_type!=MAGIC.clssDesc("SArray")) { //will never match independently of arrDim
      if (asCast) exit(-1);
      return false;
    }
    if (unitType==MAGIC.clssDesc("SArray")) { //special test for arrays
      if (o._r_unitType==MAGIC.clssDesc("SArray")) arrDim--; //an array of SArrays, make next test to ">=" instead of ">"
      if (o._r_dim>arrDim) return true; //at least one level has to be left to have an object of type SArray
      if (asCast) exit(-1);
      return false;
    }
    //no specials, check arrDim and check for standard type
    if (o._r_stdType!=stdType || o._r_dim<arrDim) { //check standard types and array dimension
      if (asCast) exit(-1);
      return false;
    }
    if (stdType!=0) return true; //array of standard-type
    //array of objects, make deep-check for class type
    clss=(SClassDesc)o._r_unitType;
    while (clss!=null) {
      if (clss==unitType) return true;
      clss=clss.parent;
    }
    if (asCast) exit(-1);
    return false;
  }
  
  public static void checkArrayStore(SArray dest, SArray newEntry) {
    //in fact newEntry is of type "Object", SArray elements are not valid always - but this is faster for _r_dim>1 which needs SArray parm
    if (dest._r_dim>1) isArray(newEntry, dest._r_stdType, dest._r_unitType, dest._r_dim-1, true); //check array
    else if (dest._r_unitType==null) {
      kernel.Viewer.println("#arrStr#");
      exit(-1);
    }
    else isInstance(newEntry, (SClassDesc)dest._r_unitType, true); //check instance
  }
  
  public static void boundException(SArray arr, int ind) {
    kernel.Viewer.println("#bound#");
    exit(-1);
  }
  
  public static void exit(int status) {
    MAGIC.inline(0xB8, 0x01, 0x00, 0x00, 0x00); //mov eax,1 (terminate process)
    MAGIC.inline(0x8B, 0x5D, 0x08);             //mov ebx,[ebp+8] (exit code)
    MAGIC.inline(0xCD, 0x80);                   //call kernel
  }
  
  private static int brk(int newEnd) {
    int tmp=0;
    MAGIC.inline(0xB8, 0x2D, 0x00, 0x00, 0x00); //mov eax,45 (brk)
    MAGIC.inline(0x8B, 0x5D, 0x08);             //mov ebx,[ebp+8] (end_data_segment)
    MAGIC.inline(0xCD, 0x80);                   //call kernel
    MAGIC.inline(0x89, 0x45, 0xFC);             //mov [ebp-4],eax
    return tmp;
  }
}

package kernel;

public class Kernel {
  private final static int[] constArray = {1, 2, 3}; //should match binary file for constArrayTest
  
  public static void main() {
    int i, esp, argc, argv;

    Viewer.print("Command line had ");
    esp=MAGIC.rMem32(MAGIC.imageBase-4);
    Viewer.print(argc=MAGIC.rMem32(esp));
    Viewer.print(" argument(s): ");
    argv=esp+4;
    for (i=0; i<argc; i++) printArg(MAGIC.rMem32(argv+(i<<2)));
    Viewer.println();
    
    ifTest(); Viewer.printChar(32); //line continued
    whileTest(); Viewer.printChar(32); //line continued
    forTest(); Viewer.printChar(32); //line continued
    charTest(); Viewer.printChar(32); //line continued
    switchTest(); Viewer.println();
    stringTest(); Viewer.println();
    newTest(); Viewer.println();
    superTest(); Viewer.println();
    constrTest(); Viewer.println(); //needs two lines
    longTest1(); //automatic new line
    longTest2(); //automatic new line
    longTest3(); Viewer.println();
    ariTest(); Viewer.println();
    castTest(); Viewer.printChar(32); //line continued by intfTest
    intfTest(); Viewer.println();
    thisTest(); Viewer.printChar(32); //line continued by nullTest
    nullTest(); Viewer.println();
    arrayTest(); Viewer.printChar(32); //line continued by constArrayTest 
    constArrayTest(); Viewer.println();
    multiDimArrTest(); Viewer.println();
    
    Viewer.println("Tests done");
  }
  
  public static void printArg(int addr) {
    int c;
    
    while ((c=(int)MAGIC.rMem8(addr++)&0xFF)!=0) Viewer.printChar(c);
    Viewer.printChar(32);
  }
  
  public static void charTest() {
    char a='a', b;
    b='b';
    Viewer.printChar((int)a);
    Viewer.printChar((int)b);
    Viewer.printChar((int)'c');
    Viewer.printChar(100);
  }
  
  public static void ifTest() {
    int a, b;

    a=3;
    b=4;
    if (a==b || a+1==b) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (b==4) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
  }
  
  public static void whileTest() {
    int a=0, b=4;
    
    while (a<b) { //1234
      Viewer.printChar((int)'1'+a++);
    }
  }
  
  public static void whileTest2() { //do not call, only check assembly!
    while (false) MAGIC.inline(0x90);
    MAGIC.inline(0x90);
    while (true) MAGIC.inline(0x90);
  }
  
  public static void forTest() {
    int a, b=4;
    
    for (a=0; a<b; a++) Viewer.printChar((int)'1'+a); //1234
  }
  
  public static void switchTest() {
    int i=3;
    long l=5l;
    
    Viewer.print("s1:");
    switch (i) {
      case 1:
      case 2:  Viewer.print("wrong<=2"); break;
      case 3:  Viewer.print("good==3"); break;
      case 4:  Viewer.print("wrong==4"); break;
      default: Viewer.print("wrong==default"); break;
    }
    Viewer.print(", s2:");
    switch (i) {
      case 1:  Viewer.print("wrong==1"); break;
      default: Viewer.print("good==default"); break;
    }
    Viewer.print(", s3:");
    switch (l) {
      case 4l: Viewer.print("early");
      case 5l: Viewer.print("goodifafter:");
      case 6l: Viewer.print("after");
    }
    Viewer.print(" done");
  }
  
  public static void stringTest() {
    String hello;
    
    MAGIC.inline(0x90);
    hello="hello world";
    MAGIC.inline(0x90);
    Viewer.print("Size of \"");
    Viewer.print(hello);
    Viewer.print("\" is: ");
    Viewer.print(hello.count);
    Viewer.printChar(0x3D); //'='
    Viewer.print(hello.length());
    Viewer.printChar(0x3D); //'='
    Viewer.print("hello world".length());
    Viewer.print(" and should be 11 like ");
    Viewer.print((hello="hello world").length());
    Viewer.print(" in \"");
    Viewer.print(hello);
    Viewer.print("\"");
  }
  
  public static void newTest() {
    TestClass t, u;
    
    Viewer.print("12 34 == ");
    t=new TestClass();
    t.set(12, 34);
    t.print();
    u=new TestClass();
    Viewer.print("; addr of o1/o2 == ");
    Viewer.printChar(32);
    Viewer.printHex((int)MAGIC.cast2Ref(t));
    Viewer.printChar(32);
    Viewer.printHex((int)MAGIC.cast2Ref(u));
  }
  
  public static void superTest() {
    TestClass t;
    
    Viewer.print("parent: ");
    t=new TestClass();
    t.toOverwrite();
    Viewer.print("; child: ");
    t=new TestClassExt();
    t.toOverwrite();
  }
  
  public static void constrTest() {
    ConstrTestClass a;
    
    Viewer.print("next line should be: 1() 2(7) 1()3() 1()4(8) 1()5() 2(9)6(9)");
    Viewer.println();
    Viewer.print("  (constr-test) ---> ");
    a=new ConstrTestClass();
    Viewer.printChar(32);
    a=new ConstrTestClass(7);
    Viewer.printChar(32);
    a=new ConstrTestClassExt1();
    Viewer.printChar(32);
    a=new ConstrTestClassExt1(8);
    Viewer.printChar(32);
    a=new ConstrTestClassExt2();
    Viewer.printChar(32);
    a=new ConstrTestClassExt2(9);
    Viewer.printChar(32);
  }
  
  public static void longTest1() {
    long l;
    
    Viewer.print("L:2/1/0/-1: ");
    l=(long)2;
    Viewer.printHexLong(l);
    Viewer.printChar(32);
    l--;
    Viewer.printHexLong(l);
    Viewer.printChar(32);
    l--;
    Viewer.printHexLong(l);
    Viewer.printChar(32);
    l--;
    Viewer.printHexLong(l);
    Viewer.printChar(32);
  }
  
  public static void longTest2() {
    long l1, l2;
    
    Viewer.print("L:3/1/-1/0: ");
    l1=(long)1;
    l2=(long)2;
    Viewer.printHexLong(l1+l2);
    Viewer.printChar(32);
    Viewer.printHexLong(l2-l1);
    Viewer.printChar(32);
    Viewer.printHexLong(l1-l2);
    Viewer.printChar(32);
    Viewer.printHexLong((l1-l2)+l1);
    Viewer.printChar(32);
  }
  
  public static void longTest3() {
    long l1, l2;
    
    Viewer.print("L:-30==0xF..FE2: ");
    l1=(long)-1;
    l2=(long)5;
    l1=l1-l2;
    Viewer.printHexLong(l1*l2); //should be -30 printing to 0xFF..FFE2
    Viewer.print(" DivRes: ");
    l1=0x1234567890l;
    if (l1/0xAl==0x1D208A5A8l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (l1/0x1234567890l==1l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (l1/0x1234567891l==0l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (l1%0xAl==0l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (l1%0x1234567890l==0l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (l1%0x1234567891l==0x1234567890l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    Viewer.printChar(32);
    l1=(long)-5;
    l2=(long)-2;
    if (l1/l2==2l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (l1%l2==-1l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    l2=2l;
    if (l1/l2==-2l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (l1%l2==-1l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    l1=5l;
    l2=-2l;
    if (l1/l2==-2l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (l1%l2==1l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    Viewer.print(" SignCmp: ");
    l1=0x00000000FFFFFFFFl;
    l2=0x000000000FFFFFFFl;
    if (l1<l2) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    Viewer.print(" ConstCmp: ");
    if (1l!=2l) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    Viewer.printChar(32);
  }
  
  public static void ariTest() {
    int a, b, c, complex;
    boolean x, y, z, r1, r2, r3, r4;
    byte b1, b2;
    short s1, s2;
    int i1, i2;
    long l1, l2;
    
    Viewer.print("ari: ");
    a=2;
    b=5;
    c=9;
    //Viewer.print("13");
    if ((a*b)+c/(b-a)==13) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    c=13-((a*b)+c/(b-a));
    //Viewer.print(" c=="); Viewer.print(c);
    if (c!=0) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    c=13;
    a=c&~b; //1101 & ~ 0101 = 1000
    //Viewer.print(" a=="); Viewer.print(a);
    if (a==8) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    c=-a;
    //Viewer.print(" c+10=="); Viewer.print(c+10);
    if (-c==a && c==-8) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    complex=42;
    complex=(complex/10)|((complex%10)<<16)|0x07300730;
    //Viewer.print(" 0x07320734=="); Viewer.print(complex);
    if (complex==0x07320734) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    Viewer.printChar(32);
    x=a*b==40; //should be true
    y=!x;
    if (x) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (y) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    z=x&&!y;
    if (z) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    Viewer.print(" .=");
    r1=z=x || boolPrinter();
    Viewer.printChar(46);
    Viewer.print(" B.=");
    r2=z=x && boolPrinter();
    Viewer.printChar(46);
    Viewer.print(" B.=");
    r3=z=!x || !boolPrinter();
    Viewer.printChar(46);
    Viewer.print(" .=");
    r4=z=!x && !boolPrinter();
    Viewer.printChar(46);
    Viewer.printChar(32);
    if (r1) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (r2) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (!r3) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (!r4) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    Viewer.printChar(32);
    r1=true;
    r2=false;
    if (r1 || r1) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (r1 || r2) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (r2 || r1) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (r2 || r2) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (r1 && r1) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (r1 && r2) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (r2 && r1) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (r2 && r2) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    Viewer.printChar(32);
    if (!TestClass.falseEmitter()) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    Viewer.printChar(32);
    a=0x13;
    b=4;
    a=(a+b-1)&~(b-1);
    if (a==0x14) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    Viewer.printChar(32);
    y=x=false;
    z=true;
    if (x==y) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (x!=z) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (!(x==y)) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    y=x==y;
    z=x==z;
    if (y) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (z) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    Viewer.printChar(32);
    l1=(long)(i1=(int)(s1=(short)(b1=(byte)0xAA)));
    l2=(long)(i2=(int)(s2=(short)(b2=(byte)0x55)));
    if (l1!=0xFFFFFFFFFFFFFFAAl) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (i1!=0xFFFFFFAA) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (s1!=(short)0xFFAA) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (b1!=(byte)0xAA) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (l2!=0x0000000000000055l) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (i2!=0x00000055) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (s2!=(short)0x0055) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (b2!=(byte)0x55) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
  }
  
  private static void directPrintInt(int val, int base, int len, int x, int y, int col) {
    int i, addr, dignum, digchr;
    long uval, ubase;

    if (len<1 || base<2 || base>16 || x<0 || x+len>80 || y<0 || y>24) return; //invalid parameter
    addr=0xB8000+((y*80+x)<<1); //address of first character
    col=(col&0xFF)<<8; //color is upper byte -> mask and shift
    uval=((long)val)&0xFFFFFFFFl;
    ubase=(long)base;
    for (i=0; i<len; i++) {
      dignum=(int)(uval%ubase);
      uval=uval/ubase;
      if (dignum==0) {
        if (i==0 || uval!=0l) digchr=48;
        else digchr=32;
      }
      else if (dignum<10) digchr=dignum+48;
      else digchr=dignum+55;
      if (i==len-1 && uval!=0l) digchr=0x3E; //">" to show "overflow"
      MAGIC.wMem16(addr+((len-i-1)<<1), (short)(col|digchr));
    }
  }
  
  private static boolean boolPrinter() {
    Viewer.printChar(66); //'B'ool called
    return true;
  }
  
  public static void castTest() {
    Kernel k, kd;
    TestClass tc, tcd;
    TestClassExt tce, tced;
    
    k=new Kernel();
    tc=new TestClass();
    tce=new TestClassExt();
    Viewer.print("castCheck: ");
    if (k instanceof TestClass) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (k instanceof TestClassExt) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (tc instanceof TestClass) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (tc instanceof TestClassExt) Viewer.printChar(87); else Viewer.printChar(103); //'W'rong or 'g'ood
    if (tce instanceof TestClassExt) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    if (tce instanceof TestClassExt) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
  }
  
  public static void intfTest() {
    Implementer i;
    TestIntf f;
    //int ai, af;
    
    i=new Implementer();
    MAGIC.inline(0x90);
    f=(TestIntf)i;
    MAGIC.inline(0x90);
    
    //ai=(int)MAGIC.addr(i);
    //af=(int)MAGIC.addr(f);
    //Viewer.printHex(ai);
    //Viewer.printChar(32);
    //Viewer.printHex(af);
    //Viewer.printChar(32); Viewer.printChar(32); Viewer.printChar(32);
    //Viewer.printHex(MAGIC.rMem32(ai));
    //Viewer.printChar(32);
    //Viewer.printHex(MAGIC.rMem32(af));
    //Viewer.printChar(32);
    //Viewer.printHex(MAGIC.rMem32(af+8));
    //Viewer.printChar(32); Viewer.printChar(32); Viewer.printChar(32);
    //Viewer.printHex(MAGIC.rMem32(MAGIC.rMem32(af+8)+0xC));
    
    Viewer.print("Like here \"");
    i.print(1);
    Viewer.print("\" there sould be a \"1\": ");
    f.print(1);
    Viewer.print(", and here too: ");
    ((TestIntf)i).print(1);
  }
  
  public static void thisTest() {
    TestClass t;
    
    Viewer.print("this: ");
    t=new TestClass();
    t.thisTest1();
    Viewer.print(" and");
    t.thisTest2();
  }
  
  public static void nullTest() {
    TestIntf intf;
    TestClass c;
    
    Viewer.print("null: ");
    intf=null;
    c=null;
    nullCallTest(intf, c);
    nullCallTest(null, null);
  }
  
  private static void nullCallTest(TestIntf i, TestClass c) {
    if (i==null && c==null) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
  }
  
  public static void arrayTest() {
    byte[] bArr;
    int[] iArr;
    TestClass[] oArr;
    int i, chk;
    
    Viewer.print("array-test new: ");
    bArr=new byte[10];
    iArr=new int[10];
    oArr=new TestClass[2];
    
    Viewer.print("done; ari: ");
    for (i=0; i<10; i++) {
      bArr[i]=(byte)(i-5);
      iArr[i]=i*10;
    }

    chk=0;
    for (i=0; i<10; i++) chk+=((int)bArr[i])+iArr[i];
    if (chk==445) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    
    for (i=0; i<10; i++) iArr[i]-=(int)bArr[i];
    chk=0;
    for (i=0; i<10; i++) chk+=iArr[i];
    if (chk==455) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    arrayParam(iArr);
    
    for (i=0; i<5; i++) iArr[9-i]=-iArr[i];
    chk=0;
    for (i=0; i<10; i++) chk+=iArr[i];
    if (chk==0) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    
    Viewer.print("; obj 1 2 3 4 == ");
    oArr[0]=new TestClass();
    oArr[1]=new TestClassExt();
    oArr[0].setDefault();
    oArr[1].setDefault();
    oArr[0].print();
    Viewer.printChar(32);
    oArr[1].print();
    Viewer.print(" done");
  }
  
  public static void arrayParam(int[] ia) {
    int chk=0, i=0;
    
    for (; i<10; i++) chk+=ia[i];
    if (chk==455) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
  }
  
  public static void constArrayTest() {
    int i;
    int[] di;
    byte[] bi;
    
    for (i=0; i<3; i++) Viewer.print(constArray[i]);
    Viewer.print("=");
    di=constArray;
    for (i=0; i<3; i++) Viewer.print(di[i]);
    Viewer.print("=");
    for (i=0; i<3; i++) Viewer.print((int)binimp.ByteData.t[i]);
    Viewer.print("=");
    bi=binimp.ByteData.t;
    for (i=0; i<3; i++) Viewer.print((int)bi[i]);
  }
  
  public static void multiDimArrTest() {
    int[][][][] arr2;
    int a, b, c, d, oc;
    
    oc=rte.DynamicRuntime.objCnt;
    arr2=new int[2][3][4][5];
    oc=rte.DynamicRuntime.objCnt-oc;
    Viewer.print("created ");
    Viewer.print(oc);
    if (oc==33) Viewer.printChar(103); else Viewer.printChar(87); //'g'ood or 'W'rong
    Viewer.print(" arrays, [0]s: ");
    Viewer.printHex((int)MAGIC.cast2Ref(arr2));
    Viewer.print(", ");
    Viewer.printHex((int)MAGIC.cast2Ref(arr2[0]));
    Viewer.print(", ");
    Viewer.printHex((int)MAGIC.cast2Ref(arr2[0][0]));
    Viewer.print(", ");
    Viewer.printHex((int)MAGIC.cast2Ref(arr2[0][0][0]));
    Viewer.print(", check: ");
    for (a=0; a<2; a++) for (b=0; b<3; b++) for (c=0; c<4; c++) for (d=0; d<5; d++)
      arr2[a][b][c][d]=((a*3+b)*4+c)*5+d;
    for (a=0; a<2; a++) for (b=0; b<3; b++) for (c=0; c<4; c++) for (d=0; d<5; d++)
      if (arr2[a][b][c][d]!=((a*3+b)*4+c)*5+d) Viewer.printChar(87); //'W'rong
    for (a=1; a>=0; a--) for (b=2; b>=0; b--) for (c=3; c>=0; c--) for (d=4; d>=0; d--)
      arr2[a][b][c][d]=-((a*3+b)*4+c)*5+d; //note: "+d" is not negated
    for (a=0; a<2; a++) for (b=0; b<3; b++) for (c=0; c<4; c++) for (d=0; d<5; d++)
      if (-arr2[a][b][c][d]!=((a*3+b)*4+c)*5-d) Viewer.printChar(87); //'W'rong
    Viewer.print(" done");
  }
}


package kernel;

public class TestClass {
  private int a, b;
  
  public void set(int ia, int ib) { a=ia; b=ib; }
  public void setDefault() { a=1; b=2; }
  public void print() { Viewer.print(a); Viewer.printChar(0x20); Viewer.print(b); }
  public static boolean falseEmitter() { return false; }
  public TestClass getThis() { return this; }
  public void thisTest1() {
    a=0;
    b=0;
    this.a=1;
    this.b=2;
    Viewer.print("thisTest: 1 2 == ");
    this.print();
    Viewer.print(" == ");
    getThis().print();
  }
  public void thisTest2() {
    TestClass me;
    me=this;
    Viewer.print(" == ");
    me.print();
    if (me==this) Viewer.print(" good");
    else Viewer.print(" wrong");
  }
  public void toOverwrite() {
    Viewer.print("parent");
  }
}

package kernel;

public class TestClassExt extends TestClass {
  public void reset() { set(0, 0); }
  public void setDefault() { set(3, 4); }
  public void toOverwrite() {
    Viewer.print("child calling parent: ");
    super.toOverwrite();
  }
}

package kernel;

public class ConstrTestClass {
  public ConstrTestClass() {
    Viewer.print("1()");
  }
  
  public ConstrTestClass(int val) {
    Viewer.print("2(");
    Viewer.print(val);
    Viewer.print(")");
  }
}

package kernel;

public class ConstrTestClassExt1 extends ConstrTestClass {
  public ConstrTestClassExt1() {
    Viewer.print("3()");
  }
  
  public ConstrTestClassExt1(int val) {
    Viewer.print("4(");
    Viewer.print(val);
    Viewer.print(")");
  }
}

package kernel;

public class ConstrTestClassExt2 extends ConstrTestClass {
  public ConstrTestClassExt2() {
    super();
    Viewer.print("5()");
  }
  
  public ConstrTestClassExt2(int val) {
    super(val);
    Viewer.print("6(");
    Viewer.print(val);
    Viewer.print(")");
  }
}

package kernel;

public interface TestIntf {
  public void print(int i);
}

package kernel;

public class Implementer implements TestIntf {
  public void print(byte b) {
    Viewer.print("wrong call (b)");
  }
  
  public void print(int i) {
    Viewer.print(i);
  }
  
  public void print(long l) {
    Viewer.print("wrong call (l)");
  }
}

package kernel;

public class Viewer {
  public static void printChar(int c) {
    MAGIC.inline(0xB8, 0x04, 0x00, 0x00, 0x00); //mov eax,4 (print string)
    MAGIC.inline(0xBB, 0x01, 0x00, 0x00, 0x00); //mov ebx,1 (handle for std-out)
    MAGIC.inline(0x8D, 0x4D, 0x08);             //lea ecx,[ebp+8] (address of string)
    MAGIC.inline(0x89, 0xDA);                   //mov edx,ebx (length of string: 1)
    MAGIC.inline(0xCD, 0x80);                   //call kernel
    //sync for debugging
    //MAGIC.inline(0xB8, 0x94, 0x00, 0x00, 0x00); //mov eax,148 (sync file)
    //MAGIC.inline(0xBB, 0x01, 0x00, 0x00, 0x00); //mov ebx,1 (handle for std-out)
    //MAGIC.inline(0xCD, 0x80);                   //call kernel
  }
  
  public static void print(int i) {
    if (i<0) {
      printChar(45);
      i=-i;
    }
    if (i==0) printChar(48);
    else {
      if (i>=10) print(i/10);
      printChar(48+i%10);
    }
  }
  
  public static void printHex(int i) {
    int v, p;
    
    for (p=0; p<8; p++) {
      v=(i>>>((7-p)<<2))&0xF;
      if (v<10) printChar(48+v); //'0'..'9'
      else printChar(55+v); //'A'..'Z'
    }
  }
  
  public static void printHexByte(int b) {
    int v, p;
    
    for (p=0; p<2; p++) {
      v=(b>>>((1-p)<<2))&0xF;
      if (v<10) printChar(48+v); //'0'..'9'
      else printChar(55+v); //'A'..'Z'
    }
  }
  
  public static void printHexLong(long l) {
    printHex((int)(l>>>32));
    printHex((int)l);
  }
  
  public static void print(String what) {
    int i;
    
    for (i=0; i<what.count; i++) printChar((int)what.value[i]);
  }
  
  public static void println() {
    printChar(10);
  }

  public static void println(String what) {
    print(what);
    printChar(10);
  }
}
