package java.lang;

import rte.SClassDesc;

public class Object {
  public SClassDesc _r_type; //fixed by compiler: reloc-position 0
}

package java.lang;

public class String {
  public char[] value;
  public int count;

  public int length() {
    return count;
  }
}

package rte;

public class SArray {
  public int length, _r_dim, _r_stdType;
  public Object _r_unitType;
}

package rte;

public class SClassDesc {
  public SClassDesc parent;
  public SIntfMap implementations;
}

package rte;

public class SIntfDesc {
}

package rte;

public class SIntfMap {
  public SIntfDesc owner;
  public SIntfMap next;
}

package rte;

public class SMthdBlock {
}

package rte;

public class DynamicRuntime {
  public static int objCnt;
  
  private final static int MEMBLOCK = 1024*1024; //1 MB per block
  private final static int BLOCKMAX = 64*1024;   //allocate objects up to 64 KB in memblock
  private static int nextFreeAddr, memFree;
  
  public static Object newInstance(int scalarSize, int relocEntries, SClassDesc type) {
    int addr, rs, size;
    Object me;
    
    //get informations
    rs=relocEntries*MAGIC.ptrSize;
    scalarSize=(scalarSize+MAGIC.ptrSize-1)&~(MAGIC.ptrSize-1);
    size=rs+scalarSize;
    if (size>BLOCKMAX) addr=allocMem(size); //allocate special block
    else { //allocate inside the pool
      if (size>memFree) {
        nextFreeAddr=allocMem(MEMBLOCK);
        memFree=MEMBLOCK;
      }
      addr=nextFreeAddr;
      memFree-=size;
      nextFreeAddr+=size;
    }
    //prepare object
    setMem32(addr, size>>2, 0); //clear memory
    me=MAGIC.cast2Obj(addr+rs); //place object
    me._r_type=type; //set type
    objCnt++;
    return me;
  }
  
  private static int allocMem(int size) {
    int result;
    
    result=brk(0);
    if (result<=0) {
      kernel.Viewer.print("Error allocating memory");
      exit(-1);
    }
    size=(size+0xFFF)&0xFFFFF000;
    brk(result+size);
    return result;
  }
  
  private static void setMem32(int addr, int cnt, int val) {
    MAGIC.inline(0xFC);             //cld
    MAGIC.inline(0x8B, 0x7D, 0x10); //mov edi,[ebp+16]
    MAGIC.inline(0x8B, 0x45, 0x08); //mov eax,[ebp+8]
    MAGIC.inline(0x8B, 0x4D, 0x0C); //mov ecx,[ebp+12]
    MAGIC.inline(0xF3, 0xAB);       //rep stosd
  }
  
  public static SArray newArray(int length, int arrDim, int entrySize, int stdType, Object unitType) {
    int scS, rlE;
    SArray me;
    
    scS=MAGIC.getInstScalarSize("SArray");
    rlE=MAGIC.getInstRelocEntries("SArray");
    if (arrDim>1 || entrySize<0) rlE+=length; //objects => rlE will become bigger
    else scS+=length*entrySize; //entrySize>0 => scS will become bigger
    me=(SArray)newInstance(scS, rlE, MAGIC.clssDesc("SArray"));
    me.length=length;
    me._r_dim=arrDim;
    me._r_stdType=stdType;
    me._r_unitType=unitType;
    return me;
  }
  
  public static void newMultArray(SArray[] parent, int curLevel, int destLevel,
      int length, int arrDim, int entrySize, int stdType, Object unitType) {
    int i;
    
    if (curLevel+1<destLevel) { //step down one level
      curLevel++;
      for (i=0; i<parent.length; i++) {
        newMultArray((SArray[])((Object)parent[i]), curLevel, destLevel,
            length, arrDim, entrySize, stdType, unitType);
      }
    }
    else { //create the new entries
      destLevel=arrDim-curLevel;
      for (i=0; i<parent.length; i++) {
        parent[i]=newArray(length, destLevel, entrySize, stdType, unitType);
      }
    }
  }
  
  public static boolean isInstance(Object o, SClassDesc dest, boolean asCast) {
    SClassDesc check;
    
    if (o==null) {
      if (asCast) return true; //null matches all
      return false; //null is not an instance
    }
    check=o._r_type;
    while (check!=null) {
      if (check==dest) return true;
      check=check.parent;
    }
    if (asCast) exit(-1);
    return false;
  }
  
  public static SIntfMap isImplementation(Object o, SIntfDesc dest, boolean asCast) {
    SIntfMap check;
    
    if (o==null) return null;
    check=o._r_type.implementations;
    while (check!=null) {
      if (check.owner==dest) return check;
      check=check.next;
    }
    if (asCast) exit(-1);
    return null;
  }
  
  public static boolean isArray(SArray o, int stdType, Object unitType, int arrDim, boolean asCast) {
    SClassDesc clss;
    
    //in fact o is of type "Object", _r_type has to be checked below - but this check is faster than "instanceof" and conversion
    if (o==null) {
      if (asCast) return true; //null matches all
      return false; //null is not an instance
    }
    if (o._r_type!=MAGIC.clssDesc("SArray")) { //will never match independently of arrDim
      if (asCast) exit(-1);
      return false;
    }
    if (unitType==MAGIC.clssDesc("SArray")) { //special test for arrays
      if (o._r_unitType==MAGIC.clssDesc("SArray")) arrDim--; //an array of SArrays, make next test to ">=" instead of ">"
      if (o._r_dim>arrDim) return true; //at least one level has to be left to have an object of type SArray
      if (asCast) exit(-1);
      return false;
    }
    //no specials, check arrDim and check for standard type
    if (o._r_stdType!=stdType || o._r_dim<arrDim) { //check standard types and array dimension
      if (asCast) exit(-1);
      return false;
    }
    if (stdType!=0) return true; //array of standard-type
    //array of objects, make deep-check for class type
    clss=(SClassDesc)o._r_unitType;
    while (clss!=null) {
      if (clss==unitType) return true;
      clss=clss.parent;
    }
    if (asCast) exit(-1);
    return false;
  }
  
  public static void checkArrayStore(SArray dest, SArray newEntry) {
    //in fact newEntry is of type "Object", SArray elements are not valid always - but this is faster for _r_dim>1 which needs SArray parm
    if (dest._r_dim>1) isArray(newEntry, dest._r_stdType, dest._r_unitType, dest._r_dim-1, true); //check array
    else if (dest._r_unitType==null) {
      kernel.Viewer.println("#arrStr#");
      exit(-1);
    }
    else isInstance(newEntry, (SClassDesc)dest._r_unitType, true); //check instance
  }
  
  public static void boundException(SArray arr, int ind) {
    kernel.Viewer.println("#bound#");
    exit(-1);
  }
  
  public static void exit(int status) {
    MAGIC.inline(0xB8, 0x01, 0x00, 0x00, 0x00); //mov eax,1 (terminate process)
    MAGIC.inline(0x8B, 0x5D, 0x08);             //mov ebx,[ebp+8] (exit code)
    MAGIC.inline(0xCD, 0x80);                   //call kernel
  }
  
  private static int brk(int newEnd) {
    int tmp=0;
    MAGIC.inline(0xB8, 0x2D, 0x00, 0x00, 0x00); //mov eax,45 (brk)
    MAGIC.inline(0x8B, 0x5D, 0x08);             //mov ebx,[ebp+8] (end_data_segment)
    MAGIC.inline(0xCD, 0x80);                   //call kernel
    MAGIC.inline(0x89, 0x45, 0xFC);             //mov [ebp-4],eax
    return tmp;
  }
}

package kernel;

public class Viewer {
  public static void printChar(int c) {
    MAGIC.inline(0xB8, 0x04, 0x00, 0x00, 0x00); //mov eax,4 (print string)
    MAGIC.inline(0xBB, 0x01, 0x00, 0x00, 0x00); //mov ebx,1 (handle for std-out)
    MAGIC.inline(0x8D, 0x4D, 0x08);             //lea ecx,[ebp+8] (address of string)
    MAGIC.inline(0x89, 0xDA);                   //mov edx,ebx (length of string: 1)
    MAGIC.inline(0xCD, 0x80);                   //call kernel
    //sync for debugging
    //MAGIC.inline(0xB8, 0x94, 0x00, 0x00, 0x00); //mov eax,148 (sync file)
    //MAGIC.inline(0xBB, 0x01, 0x00, 0x00, 0x00); //mov ebx,1 (handle for std-out)
    //MAGIC.inline(0xCD, 0x80);                   //call kernel
  }
  
  public static void print(int i) {
    if (i<0) {
      printChar(45);
      i=-i;
    }
    if (i==0) printChar(48);
    else {
      if (i>=10) print(i/10);
      printChar(48+i%10);
    }
  }
  
  public static void printHex(int i) {
    int v, p;
    
    for (p=0; p<8; p++) {
      v=(i>>>((7-p)<<2))&0xF;
      if (v<10) printChar(48+v); //'0'..'9'
      else printChar(55+v); //'A'..'Z'
    }
  }
  
  public static void printHexByte(int b) {
    int v, p;
    
    for (p=0; p<2; p++) {
      v=(b>>>((1-p)<<2))&0xF;
      if (v<10) printChar(48+v); //'0'..'9'
      else printChar(55+v); //'A'..'Z'
    }
  }
  
  public static void printHexLong(long l) {
    printHex((int)(l>>>32));
    printHex((int)l);
  }
  
  public static void print(String what) {
    int i;
    
    for (i=0; i<what.count; i++) printChar((int)what.value[i]);
  }
  
  public static void println() {
    printChar(10);
  }

  public static void println(String what) {
    print(what);
    printChar(10);
  }
}
package kernel;

public class Kernel {
  public final static int RTLD_NOW = 0x00002;
  public final static int RTLD_GLOBAL = 0x00100;

  public final static int ZPixmap = 2;

  public final static int ButtonPressMask = 0x00000004;
  public final static int ButtonPress = 4;

  public static int XOpenDisplay;
  public static int XDisplayWidth, XDisplayHeight, XDisplayDepth;
  public static int XDefaultScreen, XDefaultRootWindow;
  public static int XCreateSimpleWindow, XMapWindow, XCloseDisplay;
  public static int XDefaultGC;
  public static int XFlush;
  public static int XSelectInput, XNextEvent;
  public static int XSetForeground, XDrawLine;
  public static int XDefaultVisual, XCreateImage, XPutImage;

  public static void main() {
    int i, j, subWindowClick=0;

    Viewer.print("dlopen, dlsym, esp: ");
    Viewer.printHex(i=MAGIC.rMem32(MAGIC.imageBase-12));
    Viewer.print(", ");
    Viewer.printHex(j=MAGIC.rMem32(MAGIC.imageBase-8));
    Viewer.print(", ");
    Viewer.printHex(MAGIC.rMem32(MAGIC.imageBase-4));
    Viewer.println();

    if (i==0 || j==0) {
      Viewer.println("no library function loaded");
      return;
    }

    Viewer.print("getting handle for libX11.so.6: ");
    Viewer.printHex(i=dlopen(MAGIC.toByteArray("libX11.so.6", true), RTLD_NOW|RTLD_GLOBAL));
    Viewer.println();

    if (i==0) {
      Viewer.println("library not found");
      return;
    }

    Viewer.print("getting address of XOpenDisplay: ");
    Viewer.printHex(XOpenDisplay=dlsym(i, MAGIC.toByteArray("XOpenDisplay", true)));
    Viewer.println();

    if (XOpenDisplay==0) {
      Viewer.println("function not found");
      return;
    }
    
    if ((XDisplayWidth=dlsym(i, MAGIC.toByteArray("XDisplayWidth", true)))==0
        || (XDisplayHeight=dlsym(i, MAGIC.toByteArray("XDisplayHeight", true)))==0
        || (XDisplayDepth=dlsym(i, MAGIC.toByteArray("XDefaultDepth", true)))==0
        || (XDefaultScreen=dlsym(i, MAGIC.toByteArray("XDefaultScreen", true)))==0
        || (XDefaultRootWindow=dlsym(i, MAGIC.toByteArray("XDefaultRootWindow", true)))==0
        || (XCreateSimpleWindow=dlsym(i, MAGIC.toByteArray("XCreateSimpleWindow", true)))==0
        || (XMapWindow=dlsym(i, MAGIC.toByteArray("XMapWindow", true)))==0
        || (XCloseDisplay=dlsym(i, MAGIC.toByteArray("XCloseDisplay", true)))==0
        || (XDefaultGC=dlsym(i, MAGIC.toByteArray("XDefaultGC", true)))==0
        || (XFlush=dlsym(i, MAGIC.toByteArray("XFlush", true)))==0
        || (XSelectInput=dlsym(i, MAGIC.toByteArray("XSelectInput", true)))==0
        || (XNextEvent=dlsym(i, MAGIC.toByteArray("XNextEvent", true)))==0
        || (XSetForeground=dlsym(i, MAGIC.toByteArray("XSetForeground", true)))==0
        || (XDrawLine=dlsym(i, MAGIC.toByteArray("XDrawLine", true)))==0
        || (XDefaultVisual=dlsym(i, MAGIC.toByteArray("XDefaultVisual", true)))==0
        || (XCreateImage=dlsym(i, MAGIC.toByteArray("XCreateImage", true)))==0
        || (XPutImage=dlsym(i, MAGIC.toByteArray("XPutImage", true)))==0) {
      Viewer.println("function resolving failed");
      return;
    }

    int display=XOpenDisplay(0);
    if (display==0) {
      Viewer.println("could not open default display");
      return;
    }
    Viewer.print("dimension of default display: ");
    Viewer.print(XDisplayWidth(display, XDefaultScreen(display)));
    Viewer.print(" x ");
    Viewer.print(XDisplayHeight(display, XDefaultScreen(display)));
    Viewer.println();
    int colorDepth=XDisplayDepth(display, XDefaultScreen(display));
    Viewer.print("color depth: ");
    Viewer.print(colorDepth);
    Viewer.println(" bits per pixel");

    int mainWindow=XCreateSimpleWindow(display, XDefaultRootWindow(display), 0, 0, 
      200, 100, 0, 0, 0);
    if (mainWindow==0) {
      Viewer.println("could not create main window");
      return;
    }
    Viewer.print("created main window, ");
    int mainGC=XDefaultGC(display, 0);
    Viewer.print("got gc, ");
    int subWindow=XCreateSimpleWindow(display, mainWindow, 10, 10,
      50, 50, 0, 0, 0xFFFFFF);
    if (subWindow==0) {
      Viewer.println("could not create sub window");
      return;
    }
    Viewer.print("created sub window, ");
    XMapWindow(display, mainWindow);
    XMapWindow(display, subWindow);
    Viewer.print("mapped, ");
    XFlush(display);
    Viewer.println("flushed");

    XSelectInput(display, mainWindow, ButtonPressMask);
    XSelectInput(display, subWindow, ButtonPressMask);
    Viewer.println("selected input, waiting for events...");

    int image=0;
    int[] imageData=null;
    XEvent event;
    XButtonEvent buttonEvent;
    event=(XEvent)MAGIC.cast2Struct(MAGIC.addr((new byte[96])[0]));
    buttonEvent=(XButtonEvent)MAGIC.cast2Struct(MAGIC.cast2Ref(event));
    while (true) {
      XNextEvent(display, event);
      if (event.type==ButtonPress) {
        if (buttonEvent.window==mainWindow) {
          Viewer.println("drawing line after main window click");
          XSetForeground(display, mainGC, 0xFFFFFF);
          XDrawLine(display, mainWindow, mainGC, 70, 20, 150, 40);
          XFlush(display);
        }
        else if (buttonEvent.window==subWindow) {
          switch (subWindowClick++) {
            case 0: //try to draw image
              if (colorDepth!=24 && colorDepth!=32) Viewer.println("current demo requires 24 or 32 bit color depth on screen");
              else {
                Viewer.println("drawing colored image after sub window click");
                imageData=new int[20*20];
                for (i=0; i<20; i++) {
                  imageData[i+i*20]=0xFFFFFF;
                  imageData[i]=0xFF00FF;
                  imageData[i+20]=0xFF00FF;
                  imageData[i+18*20]=0x00FF00;
                  imageData[i+19*20]=0x00FF00;
                }
                image=XCreateImage(display, XDefaultVisual(display, XDefaultScreen(display)), colorDepth, ZPixmap, 0,
                  MAGIC.addr(imageData[0]), 20, 20, 8, 80);
                if (image==0) Viewer.println("could not create image");
                else {
                  XPutImage(display, subWindow, mainGC, image, 0, 0, 5, 5, 20, 20);
                  XFlush(display);
                }
              }
              break;
            case 1:
              Viewer.println("modifying image");
              if (image!=0) { //image valid, modify and update
                for (i=5; i<15; i++) imageData[19-i+i*20]=0xFFFFFF;
                XPutImage(display, subWindow, mainGC, image, 5, 5, 10, 10, 10, 10);
                XFlush(display);
                break;
              }
              //no break: terminate application
            default:
              Viewer.println("closing display after sub window click");
              XCloseDisplay(display);
              return;
          }
        }
      }
    }
  }

  public static int dlopen(byte[] libNameZ, int flags) {
    int libAddr=MAGIC.addr(libNameZ[0]);

    MAGIC.inline(0xFF, 0x75); MAGIC.inlineOffset(1, flags);
    MAGIC.inline(0xFF, 0x75); MAGIC.inlineOffset(1, libAddr);
    MAGIC.inline(0xFF, 0x15); MAGIC.inline32(MAGIC.imageBase-12);
    MAGIC.inline(0x89, 0x45); MAGIC.inlineOffset(1, libAddr);
    MAGIC.inline(0x5B, 0x5B);
    return libAddr;
  }

  public static int dlsym(int libHandle, byte[] funcNameZ) {
    int funcAddr=MAGIC.addr(funcNameZ[0]);

    MAGIC.inline(0xFF, 0x75); MAGIC.inlineOffset(1, funcAddr);
    MAGIC.inline(0xFF, 0x75); MAGIC.inlineOffset(1, libHandle);
    MAGIC.inline(0xFF, 0x15); MAGIC.inline32(MAGIC.imageBase-8);
    MAGIC.inline(0x89, 0x45); MAGIC.inlineOffset(1, funcAddr);
    MAGIC.inline(0x5B, 0x5B);
    return funcAddr;
  }

  public native static int XOpenDisplay(int no);
  public native static int XDisplayWidth(int display, int screen);
  public native static int XDisplayHeight(int display, int screen);
  public native static int XDisplayDepth(int display, int screen);
  public native static int XDefaultScreen(int display);
  public native static int XDefaultRootWindow(int display);
  public native static int XCreateSimpleWindow(int display, int root, int x, int y,
      int width, int height, int unknown, int color1, int color2);
  public native static void XMapWindow(int display, int window);
  public native static void XCloseDisplay(int display);
  public native static int XDefaultGC(int display, int unknown);
  public native static void XFlush(int display);
  public native static void XSelectInput(int display, int window, int mask);
  public native static void XNextEvent(int display, XEvent event);
  public native static void XSetForeground(int display, int gc, int color);
  public native static void XDrawLine(int display, int window, int gc,
      int x1, int y1, int x2, int y2);
  public native static int XDefaultVisual(int display, int screen);
  public native static int XCreateImage(int display, int visual, int depth, int format,
      int offset, int data, int width, int height, int pad, int bytes_per_line);
  public native static void XPutImage(int display, int window, int gc, int image,
      int srcX, int srcY, int dstX, int dstY, int width, int height);
}

package kernel;

public class XEvent extends STRUCT {
  int type;
}

package kernel;

public class XButtonEvent extends STRUCT {
  int type; /* has to be ButtonPress */
  int serial; /* # of last request processed by server */
  int send_event;  /* true if this came from a SendEvent request */
  int display;  /* Display the event was read from */
  int window;          /* "event" window it is reported relative to */
  int root;          /* root window that the event occurred on */
  int subwindow;  /* child window */
  int time;    /* milliseconds */
  int x, y;    /* pointer x, y coordinates in event window */
  int x_root, y_root;  /* coordinates relative to root */
  int state;  /* key or button mask */
  int button;  /* detail */
  int same_screen;  /* same screen flag */
}

